#
# Example output when the web interface changes item 3 and the email
# (non pgp) interface changes item 4:
#
# tx_SourceCheckAudit(3) pre db.tx_Source: web
# tx_SourceCheckAudit(4) pre db.tx_Source: email
# tx_SourceCheckAudit(3) post db.tx_Source: web
# tx_SourceCheckAudit(4) post db.tx_Source: email
# tx_SourceCheckReact(4) pre db.tx_Source: email
# tx_SourceCheckReact(4) post db.tx_Source: email
# tx_SourceCheckReact(3) pre db.tx_Source: web
# tx_SourceCheckReact(3) post db.tx_Source: web
#
# Note that the calls are interleaved, but the proper
# tx_Source is associated with the same ticket.

import time as time
import os

def tx_SourceCheckAudit(db, cl, nodeid, newvalues):
    ''' An auditor to print the value of the source of the
        transaction that trigger this change. The sleep call
        is used to delay the transaction so that multiple changes will
        overlap. The expected output from this detector are 2 lines
        with the same value for tx_Source. Tx source is:
          unset - reported when using a script/roundup-admin
          cgi - reported when using any web based technique
          email - reported when using any email based technique
          email/pgp - reported when email based technique uses pgp signed emails
    '''
    nonce = os.urandom(10).encode('hex')

    print "tx_SourceCheckAudit(%s) pre db.tx_Source: %s, nonce: %s"%(nodeid, db.tx_Source, nonce)

    time.sleep(10)

    newvalues['tx_Source'] = 'arrived via: %s %s' % (db.tx_Source, nonce)

    print "tx_SourceCheckAudit(%s) post db.tx_Source: %s, nonce %s"%(nodeid, db.tx_Source, nonce)

    # example use for real to prevent a change from happening if it's
    # submited via email
    #
    # if db.tx_Source == "email":
    #    raise Reject, 'Change not allowed via email'

def tx_SourceCheckReact(db, cl, nodeid, oldvalues):
    ''' An reactor to print the value of the source of the
        transaction that trigger this change. The sleep call
        is used to delay the transaction so that multiple changes will
        overlap. The expected output from this detector are 2 lines
        with the same value for tx_Source. Tx source is:
          unset - reported when using a script/roundup-admin
          web - reported when using any web based technique
          email - reported when using any email based technique
          email/pgp - reported when email based technique uses pgp signed emails
    '''

    # get the tx_Source property using cl.get(nodeid, 'messages')

    print "tx_SourceCheckReact(%s) pre db.tx_Source: %s, recorded: %s"%(nodeid, db.tx_Source, cl.get(nodeid, 'tx_Source'))
    time.sleep(10)
    print "tx_SourceCheckReact(%s) post db.tx_Source: %s, recorded: %s"%(nodeid, db.tx_Source, cl.get(nodeid, 'tx_Source'))

def init(db):
    db.issue.audit('set', tx_SourceCheckAudit)
    db.issue.audit('create', tx_SourceCheckAudit)
    db.issue.react('set', tx_SourceCheckReact)
    db.issue.react('create', tx_SourceCheckReact)

    db.msg.audit('create', tx_SourceCheckAudit)
